// This is a file to test parsing using the JAXP-1.1
import javax.xml.parsers.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.xml.sax.SAXException;
import java.io.IOException;
import org.xml.sax.InputSource;
import java.io.*;
import java.awt.Dimension;

class FoXQReader {
    DocumentBuilder builder;
    DocumentBuilderFactory factory;
    Document document;
    AbstractComponent component;
 
    public FoXQReader(InputStream source){
	factory = DocumentBuilderFactory.newInstance();
	try {
	    builder = factory.newDocumentBuilder();
	    document = builder.parse(new InputSource(source));
	} catch (SAXException se){
	    System.out.println("We have a SAX exception here!!");
	    System.out.println(" Exception " + se.getException());
	    System.out.println(" Message   " + se.getMessage());
	} catch (IOException ioe){
	    System.out.println("We have an IOException " + "\n" + ioe.getMessage());
	} catch (ParserConfigurationException pce){
	    System.out.println("We have a configuration problem here!!");
	}
    }


    public AbstractComponent returnComponent(){
	//System.out.println("***********************");
	return this.returnComponent(document);
    }

    public AbstractComponent returnComponent(Node node){
	switch ( node.getNodeType()){
	case Node.DOCUMENT_NODE:
	    //System.out.println("<xml version=\"1.0\">\n");
	    Document doc = (Document) node;
	    component = new DFolderComponent();
	    component.setSize(new Dimension(750,665));
	    component.setName(doc.getNodeName());
	    component.toggleSelect();
	    component.add(returnComponent(doc.getDocumentElement()));
	    break;

	case Node.ELEMENT_NODE:
	    String name = node.getNodeName();
	    //System.out.print("<<" + name);
	    if (name.equals("xml")) {
		NodeList children = node.getChildNodes();
		if (children != null){
		    //System.out.println("Going for children from xml # " + children.getLength());
		    //AbstractComponent temp;
		    for (int i =0 ; i < children.getLength(); i++){
			if(children.item(i).getNodeType() == Node.ELEMENT_NODE ){
			    //System.out.println("child is element node(xml):" + children.item(i).getNodeName());
			    component.add(returnComponent(children.item(i)));
			    //System.out.println("Done adding card for :" + children.item(i).getNodeName());
			} else if (children.item(i).getNodeType() == Node.CDATA_SECTION_NODE) {
			    //System.out.println("child is data");
			} else if (children.item(i).getNodeType() == Node.TEXT_NODE) {
			    //System.out.println("child is texton" + children.item(i).getNodeValue());
			}
		    }

		}		
		//System.out.println("End of processing children from xml");
		component.toggleSelect();
	    } else {

		//System.out.println(">>" + name + " value= " + node.getNodeValue());
		NodeList children = node.getChildNodes();
		DCardComponent card;
		if (children != null){
		    //System.out.println("Going for children from non-xml #" + node.getNodeName());
		    card = new DCardComponent();
		    card.toggleSelect();
		    card.setName(node.getNodeName());
		    for (int i =0 ; i < children.getLength(); i++){
			if(children.item(i).getNodeType() == Node.ELEMENT_NODE ){
			    //System.out.println("child is element node:" + children.item(i).getNodeName());
			    card.add(returnComponent(children.item(i)));
			} else if (children.item(i).getNodeType() == Node.CDATA_SECTION_NODE) {
			    //System.out.println("child is data");
			} else if (children.item(i).getNodeType() == Node.TEXT_NODE) {
			    //System.out.println("child is texty " + children.item(i).getNodeValue());
			    if (! children.item(i).getNodeValue().trim().equals("")) {
				//System.out.println("** Not a tab " + children.item(i).getNodeValue().length());
				card.add(returnComponent(children.item(i)));
			    }
			}
		    }
		    //System.out.println("End of processing children from non-xml element :" + card.getName());
		    card.toggleSelect();
		    return card;
		}
	    }
	    //System.out.println("returning the card component");
	    return component;
		
	    
	case Node.TEXT_NODE:
	    DElementComponent element = new DElementComponent();
	    //System.out.println("Text " + node.getNodeName() + " " + node.getNodeValue());
	    element.setContent(node.getNodeValue());
	    return element;

       /*
	case Node.CDATA_SECTION_NODE:
	    // print the textual data
	    System.out.println("CData " + node.getNodeName() + " " + node.getNodeValue());
	    //return node.getNodeValue();
	    break;

	case Node.PROCESSING_INSTRUCTION_NODE:
	    // print the processing instruction
	    break;

	case Node.ENTITY_REFERENCE_NODE:
	    //print the entity referenece
	    break;

	case Node.DOCUMENT_TYPE_NODE:
	    //print the DTD declaration
	    break;
       */
	}
	component.rearrange();
	return component;
       
    } 


    public static void printNode(Node node, String indent){
	switch ( node.getNodeType()){
	case Node.DOCUMENT_NODE:
	    //System.out.println("<xml version=\"1.0\">\n");
	    Document doc = (Document) node;
	    printNode(doc.getDocumentElement(),"");
	    break;

	case Node.ELEMENT_NODE:
	    String name = node.getNodeName();
	    //System.out.print(indent + "<" + name);
	    NamedNodeMap attributes = node.getAttributes();
	    for (int i = 0 ; i< attributes.getLength(); i++){
		Node current = attributes.item(i);
		//System.out.print(" " + current.getNodeName() + "=\"" + current.getNodeValue() + "\"");
	    }
	    //System.out.println(">");
	    NodeList children = node.getChildNodes();
	    if (children != null){
		for (int i =0 ; i < children.getLength(); i++){
		    printNode(children.item(i), indent + " ");
		}
	    }
	    //System.out.println(indent + "</" + name + ">");
	    break;
	    
	case Node.TEXT_NODE:

	case Node.CDATA_SECTION_NODE:
	    // print the textual data
	    //System.out.println(indent + node.getNodeValue());
	    break;

	case Node.PROCESSING_INSTRUCTION_NODE:
	    // print the processing instruction
	    break;

	case Node.ENTITY_REFERENCE_NODE:
	    //print the entity referenece
	    break;

	case Node.DOCUMENT_TYPE_NODE:
	    //print the DTD declaration
	    break;

	}
     
    } 

}




